/*
 *    Copyright (c) 2003, Cipherica Labs. All rights reserved.
 *    See enclosed license.txt for redistribution information.
 *
 *    $Id: misc.c,v 1.2 2003/04/04 19:56:53 alex Exp $
 */

#include "x4/crypto/misc.h"
#include "x4/crypto/dh.h"
#include "x4/crypto/random.h"

void x4_dh_data_free(x4s_dh_data * dh)
{
  x4_assert(dh);
///jdd- try to add sanity checks so we don't crash
  if( (dh->g.data != 0) && (dh->g.len !=0) && (dh->g.len < 4000) ) 
  	x4_buf_free(&dh->g);
//  else
//	printf("Problem dh.g free\n");
  if( (dh->p.data != 0) && (dh->p.len !=0) && (dh->p.len < 4000) ) 
  	x4_buf_free(&dh->p);
//  else
//	printf("Problem dh.p free\n");
  if( (dh->x.data != 0) && (dh->x.len !=0) && (dh->x.len < 4000) ) 
  	x4_buf_free(&dh->x);
//  else
//	printf("Problem dh.x free\n");
  if( (dh->gx.data != 0) && (dh->gx.len !=0) && (dh->gx.len < 4000) ) 
  	x4_buf_free(&dh->gx);
//  else
//	printf("Problem dh.gx free\n");
  if( (dh->gy.data != 0) && (dh->gy.len !=0) && (dh->gy.len < 4000) ) 
  	x4_buf_free(&dh->gy);
//  else
//	printf("Problem dh.gy free\n");
  if( (dh->gxy.data != 0) && (dh->gxy.len !=0) && (dh->gxy.len < 4000) ) 
  	x4_buf_free(&dh->gxy);
//  else
//	printf("Problem dh.gxy free\n");
}

/*  */
void x4_dh_initiate(x4s_dh_data * dh)
{
  x4_assert(dh);
  x4_assert(dh->g.len && dh->p.len);
  
  /* generate private value */
  x4_buf_resize(&dh->x, dh->p.len);
  x4_randomb(&dh->x);

  /* generate public DH key */
  x4_buf_free(&dh->gx);
  dh->gx = x4_dh_publicb(&dh->g, &dh->p, &dh->x);
  
  x4_assert(dh->gx.len);

  if (dh->gx.len < dh->p.len)
    x4_buf_prepend(&dh->gx, 0, dh->p.len - dh->gx.len);
}

/*  */
bval x4_dh_complete(x4s_dh_data * dh)
{
  x4_assert(dh);
  x4_assert(dh->g.len && dh->p.len && dh->x.len);
  x4_assert(dh->gx.len && dh->gy.len);

  /* generate shared DH key */
  x4_buf_free(&dh->gxy);
  dh->gxy = x4_dh_sharedb(&dh->g, &dh->p, &dh->x, &dh->gx, &dh->gy);

  return dh->gxy.len != 0;
}
