/*  Copyright (c) 2005, Applied Security, Inc.
    Jared DeMott, jdemott@appliedsec.com, www.appliedsec.com

    This file is part of ikefuzz.

    Ikefuzz is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    CmdLine is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ikefuzz; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/*
Fuzz IKE servers (such as iskampd)
RETURN VALS: -2=unhandled et, -1=unhandled seqno, 0=nothing done, 1=fuzz data inserted
*/

#include "x4/misc/FUZZ.h"

/*The following globals are used to index to particular tests.  Test can be written 
but not included in the Tests_x array.  If you don't want to fuzz a particular leg
put a bogus test number in such as 0.  A zero (or bogus test num) should end the list.*/
uint16 COUNT[3]={0,0,0};  //used to init each leg - don't change
int Index_Zero=0;			
char Tests_Zero[]={1,2,3,4,5,0};	//5 tests defined
//char Tests_Zero[]={0};	//comment above and uncomment this one if no zero pkt tests desired

int Index_Two=0;			
char Tests_Two[]={1,0};  //1 tests defined
//char Tests_Two[]={0};  //comment above and uncomment this one if no 2nd pkt tests desired

int Index_Four=0;			
char Tests_Four[]={1,2,3,4,5,0};  //5 tests
//char Tests_Four[]={0};  //comment above and uncomment this one if no 4th pkt tests desired

uint16 COUNT_ph2[2]={0,0};  //used to init each leg - don't change
int Index_ph2_Zero=0;			
char Tests_ph2_Zero[]={1,2,3,4,5,6,7,8,9,0};  //9 tests
//char Tests_ph2_Zero[]={0};  //comment above and uncomment this one if no ph2 0 tests desired

int Index_ph2_Two=0;			
char Tests_ph2_Two[]={1,2,3};  //2 tests and a thrid to exit
//char Tests_ph2_Two[]={3};  //comment above and uncomment this one if no ph2 2nd tests desired


int FUZZ(x4s_ike_exchange * xchg, unsigned int *testNum, unsigned int *testCount)
{
	x4s_buf * Pkt=&xchg->out.pkt;
	uint8 * Data=xchg->out.pkt.data;
	uint8 * End=xchg->out.pkt.end;
	size_t Len=xchg->out.pkt.len;

	uint8 * tdata;
	uint8 tlen_8;
	uint16 tlen_16;
	uint32 tlen_32;
	int i;
	int KNextend=0;  //0=nonce was extended; 1=key was extended

 if(xchg->et == 2)
 {
   switch(xchg->seqno)
   {
     case 0:
	switch(Tests_Zero[Index_Zero])
	{
		case 1:  /*pkt0 - Bad liftypes, duratons, and other transform types*/
			if(COUNT[0]==0)
				printf("******PKT0 [funky lifetypes, durations, and other transform types] tests Beginning.******\n");	
			COUNT[0]++;
	
			///resize pkt and reset ptrs
			if (COUNT[0]<100)
				x4_buf_resize(Pkt, ((8*COUNT[0])+112) );
			else if (COUNT[0]>100 && COUNT[0]<200)
				x4_buf_resize(Pkt, (((10*COUNT[0]))/2+112) );
			else
			{
				tlen_8=rand()&0xff;
				x4_buf_resize(Pkt, (4*tlen_8)+112 );
			}
			Data=Pkt->data;
			Len=Pkt->len;
			*((uint32 *)Data+6)=htonl(Len); //fix total len
			if (COUNT[0]<100)
			{
				*((uint16 *)Data+15)=htons( 44+(8*COUNT[0]) ); //fix SA len
				*((uint16 *)Data+21)=htons( 32+(8*COUNT[0]) ); //fix proposal len
				*((uint16 *)Data+25)=htons( 24+(8*COUNT[0]) ); //fix transform len
				///add bytes
				tdata=malloc(Len);
				memcpy(tdata, Data, 72);
				for(i=0; i<(8*COUNT[0]); i+=8)
				{
					memcpy(tdata+72+i, "\x80\x0b\x00",3);
					tlen_8=rand()&0x7f;
					memcpy(tdata+72+3+i, (uint8 *)&tlen_8, 1);
					memcpy(tdata+72+4+i, "\x80\x0c",2);
					tlen_16=rand()&0xffff;
					memcpy(tdata+72+6+i, (uint8 *)&tlen_16, 2);
				}
				memcpy(tdata+72+i, Data+72, 40);
				memcpy(Data, tdata, Len);
				free(tdata);
			}
			else if (COUNT[0]>100 && COUNT[0]<200)
			{
				*((uint16 *)Data+15)=htons( 44+((10*COUNT[0])/2) ); //fix SA len
				*((uint16 *)Data+21)=htons( 32+((10*COUNT[0])/2) ); //fix proposal len
				*((uint16 *)Data+25)=htons( 24+((10*COUNT[0])/2) ); //fix transform len
				///add bytes
				tdata=malloc(Len);
				memcpy(tdata, Data, 72);
				for(i=0; i<((10*COUNT[0])/2); i+=5)
				{
					memcpy(tdata+72+i, "\x80\x0b\x00",3);
					tlen_8=rand()&0x7f;
					memcpy(tdata+72+3+i, (uint8 *)&tlen_8, 1);
					memcpy(tdata+72+4+i, "\x80\x0c",2);
					tlen_32=rand()&0xffffffff;
					memcpy(tdata+72+6+i, (uint8 *)&tlen_32, 4);
				}
				memcpy(tdata+72+i, Data+72, 40);
				memcpy(Data, tdata, Len);
				free(tdata);
			}
			else
			{
				*((uint16 *)Data+15)=htons( 44+(4*tlen_8) ); //fix SA len
				*((uint16 *)Data+21)=htons( 32+(4*tlen_8) ); //fix proposal len
				*((uint16 *)Data+25)=htons( 24+(4*tlen_8) ); //fix transform len
				///add bytes
				tdata=malloc(Len);
				memcpy(tdata, Data, 72);
				for(i=0; i<(4*tlen_8); i+=4)
				{
					memcpy(tdata+72+i, "\x80",1);
					tlen_32=rand()&0xffffffff;
					memcpy(tdata+72+1+i, (uint8 *)&tlen_32, 3);
				}
				memcpy(tdata+72+i, Data+72, 40);
				memcpy(Data, tdata, Len);
				free(tdata);

			}

			*testNum=Tests_Zero[Index_Zero];
			*testCount=COUNT[0];
			if(COUNT[0]==300)  //set to 300 for best results
			{
				printf("************PKT0 [funky life types and durations] tests complete.************\n");
				Index_Zero++;
				COUNT[0]=0;
			}
			return 1;
		case 2:  /*pkt0 - Messing with main length*/
			if(COUNT[0]==0)
				printf("******PKT0 [bad hdr len] tests Beginning.******\n");	
			COUNT[0]++;
	
			i=(rand()&0xff);  //33% len=0; 33% len=ffffffff; 33%=rand len
			if (i<85)
				*((uint32 *)Data+6)=0x00000000;
			else if(i>85 && i<170)
				*((uint32 *)Data+6)=0xffffffff;
			else
				*((uint32 *)Data+6)=(rand()&0xffffffff);

			*testNum=Tests_Zero[Index_Zero];
			*testCount=COUNT[0];
			if(COUNT[0]==100)  //set to 100 for best results
			{
				printf("************PKT0 [bad hdr len] tests complete.************\n");
				Index_Zero++;
				COUNT[0]=0;
			}
			return 1;
		case 3:  /*pkt0 - Randomize 3 bytes hdr, SA, or VendorID*/
			if(COUNT[0]==0)
				printf("******PKT0 [byte flip] tests Beginning.******\n");	
			COUNT[0]++;
			*(Data+(rand()&52))=rand()&0xff;
			*(Data+(rand()&82))=rand()&0xff;
			*(Data+(rand()&112))=rand()&0xff;
			*testNum=Tests_Zero[Index_Zero];
			*testCount=COUNT[0];
			if(COUNT[0]==200)  //set to 200 for best results
			{
				printf("************PKT0 [byte flip] tests complete.************\n");
				Index_Zero++;
				COUNT[0]=0;
			}
			return 1;
		case 4:  /*pkt0-adding Vendor IDs or add size to existing VendorID*/
			if(COUNT[0]==0)
				printf("******PKT0 [VendorIDs] tests Beginning.******\n");	
			COUNT[0]++;
			///resize pkt and reset ptrs
			x4_buf_resize(Pkt, ((20*COUNT[0])+112) );
			Data=Pkt->data;
			Len=Pkt->len;
			///fix total len
			*((uint32 *)Data+6)=htonl(Len);
			if( (rand()&0xff) >127)  //inc vendor len
			{
				*((uint16 *)Data+47)=htons(Len-92); //fix VendorLen
				for(i=112;i<Len;i++)
					*(Data+i)=(rand() & 0xff);  //tack some rand data on end
			}
			else  //add vendors to pkt
			{
				for(i=92;i<Len;i=i+20)
					memcpy(Data+i, Data+72, 20);
				*(Data+(Len-20))=0x00;
			}
			*testNum=Tests_Zero[Index_Zero];
			*testCount=COUNT[0];
			if(COUNT[0]==90)  //set to 90 for best results
			{
				printf("************PKT0 [VendorIDs] tests complete.************\n");
				Index_Zero++;
				COUNT[0]=0;
			}
			return 1;
		case 5:  /*pkt0-flip SA and Vendor positions, 75% inc SA lens*/
			if(COUNT[0]==0)
			{
				printf("******PKT0 [SA & Vendor flip] tests Beginning.******\n");	
			}
			COUNT[0]++;
			///swith SA and vendor positions
			tdata=malloc(Len);
			memcpy(tdata, Data, 28);
			memcpy(tdata+28, Data+72, 40);
			memcpy(tdata+68, Data+28, 44);
			memcpy(Data, tdata, Len);
			free(tdata);
			*(Data+16)=0x0d;  // sa to vendor
			*(Data+48)=0x01;  // none to sa
			*(Data+68)=0x00;  // vendor to none
			///75% of the time we'll increase SA size
			if( (rand()&0xff) > 64)
			{
				///resize pkt and reset ptrs
				x4_buf_resize(Pkt, ((16*COUNT[0])+112) );
				Data=Pkt->data;
				Len=Pkt->len;
				///fix total len
				*((uint32 *)Data+6)=htonl(Len);
				*((uint16 *)Data+35)=htons(Len-68); //fix SALen
				for(i=72;i<Len;i++)
					*(Data+i)=(rand() & 0xff);  //fill with rand data
			}
			*testNum=Tests_Zero[Index_Zero];
			*testCount=COUNT[0];
			if(COUNT[0]==100) //set to 100
			{
				printf("************PKT0 [SA and Vendor flip] tests complete.************\n");
				Index_Zero++;
				COUNT[0]=0;
			}
			return 1;
		default:
			return 0;	
	}
     case 2:
	switch(Tests_Two[Index_Two])
	{
		case 1:  /*pkt2- 50% of the time flip nonce and key around. INC one of the two*/
			if(COUNT[1]==0)
			{
				printf("******PKT2 [key+nonce] tests Beginning.******\n");	
			}
			COUNT[1]++;
			///resize pkt and reset ptrs
			if(COUNT[1]>100)
				x4_buf_resize(Pkt, ((16*(COUNT[1]/2))+180) );
			else
				x4_buf_resize(Pkt, ((16*COUNT[1])+180) );
				
			Data=Pkt->data;
			Len=Pkt->len;
			*((uint32 *)Data+6)=htonl(Len); //fix total Len
			//Extend either key or nonce
			if( (rand()&0xff) > 127 )
			{
				*((uint16 *)Data+15)=htons(Len-48); //fix key len
				tdata=malloc(Len); //tmp data holding
				memcpy(tdata, Data, 160); //cpy up to before insert			
				for(i=160;i<(Len-20);i++) //fill with rand to total size
					*(tdata+i)=(rand() & 0xff);
				memcpy(tdata+i, Data+160, 20); //tack on 2nd half
				memcpy(Data, tdata, Len); //cpy finished buf to correct ptr
				free(tdata);  //free temp buf
				KNextend=1;
			}
			else
			{
				*((uint16 *)Data+81)=htons(Len-160); //fix nonce len
				for(i=180;i<Len;i++) //fill with rand to total size
					*(Data+i)=(rand() & 0xff);
				KNextend=0;
			}
			//flip-flop nonce and key (50%)
			if( (rand()&0xff) > 127 )
			{
				if(KNextend==0)
				{
					*(Data+16)=0x0a;
					*(Data+28)=0x00;	
					*(Data+160)=0x04;
					tdata=malloc(Len); //tmp data holding
					memcpy(tdata, Data, 28);
					memcpy(tdata+28, Data+160, Len-160 );
					memcpy(tdata+28+(Len-160), Data+28, 132);
					memcpy(Data, tdata, Len);
					free(tdata);
				}
				if(KNextend==1)
				{
					*(Data+16)=0x0a;
					*(Data+28)=0x00;	
					*(Data+(Len-20))=0x04;
					tdata=malloc(Len); //tmp data holding
					memcpy(tdata, Data, 28);
					memcpy(tdata+28, Data+(Len-20), 20 );
					memcpy(tdata+48, Data+28, (Len-48));
					memcpy(Data, tdata, Len);
					free(tdata);
				}
			}
			*testNum=Tests_Two[Index_Two];
			*testCount=COUNT[1];
			if(COUNT[1]==200) //set to 200
			{
				printf("************PKT2 [fuzz key+nonce] tests complete.************\n");
				Index_Two++;
				COUNT[1]=0;
			}
			return 1;
		default:
			return 0;	
	}
     case 4:
	switch(Tests_Four[Index_Four])
	{
		case 1:  /*pkt4(ID protetion)-mess with IPSEC FQDN and the '@'*/
			if(COUNT[2]==0)
				printf("******PKT4(ID Protection) [USER FQDN -odd '@'] tests Beginning.******\n");	
			COUNT[2]++;
			///resize pkt and reset ptrs
			if (COUNT[2]<4)
				x4_buf_resize(Pkt, (64+60) );
			else if (COUNT[2]>3 && COUNT[2]<7)
				x4_buf_resize(Pkt, (128+60) );
			else if (COUNT[2]>6 && COUNT[2]<10)
				x4_buf_resize(Pkt, (256+60) );
			else if (COUNT[2]>9 && COUNT[2]<13)
				x4_buf_resize(Pkt, (512+60) );
			else
				x4_buf_resize(Pkt, (1024+60) );
			Data=Pkt->data;
			Len=Pkt->len;
			*((uint32 *)Data+6)=htonl(Len);  //fix total len
			*((uint16 *)Data+15)=htons(Len-52);  //fix total len
			*(Data+32)=0x03; //set ID type to FQDN
			tdata=malloc(Len); //tmp data holding
			memcpy(tdata, Data, 36); //cpy up to before insert			
			if (COUNT[2]<4)
			{
				for(i=0;i<64;i++) //fill with A's
					*(tdata+36+i)=0x41;
				memcpy(tdata+36+i-4, ".com", 4);
				if(COUNT[2]==1)
					*(tdata+36)='@';
				else if(COUNT[2]==2)
					*(tdata+36+32)='@';
				else if(COUNT[2]==3)
					*(tdata+36+63)='@';
			}
			else if (COUNT[2]>3 && COUNT[2]<7)
			{
				for(i=0;i<128;i++) //fill with A's
					*(tdata+36+i)=0x41;
				memcpy(tdata+36+i-4, ".com", 4);
				if(COUNT[2]==4)
					*(tdata+36)='@';
				else if(COUNT[2]==5)
					*(tdata+36+64)='@';
				else if(COUNT[2]==6)
					*(tdata+36+127)='@';
			}
			else if (COUNT[2]>6 && COUNT[2]<10)
			{
				for(i=0;i<256;i++) //fill with A's
					*(tdata+36+i)=0x41;
				memcpy(tdata+36+i-4, ".com", 4);
				if(COUNT[2]==7)
					*(tdata+36)='@';
				else if(COUNT[2]==8)
					*(tdata+36+128)='@';
				else if(COUNT[2]==9)
					*(tdata+36+255)='@';
			}
			else if (COUNT[2]>9 && COUNT[2]<13)
			{
				for(i=0;i<512;i++) //fill with A's
					*(tdata+36+i)=0x41;
				memcpy(tdata+36+i-4, ".com", 4);
				if(COUNT[2]==10)
					*(tdata+36)='@';
				else if(COUNT[2]==11)
					*(tdata+36+256)='@';
				else if(COUNT[2]==12)
					*(tdata+36+511)='@';
			}
			else
			{
				for(i=0;i<1024;i++) //fill with A's
					*(tdata+36+i)=0x41;
				memcpy(tdata+36+i-4, ".com", 4);
				if(COUNT[2]==13)
					*(tdata+36)='@';
				else if(COUNT[2]==14)
					*(tdata+36+512)='@';
				else if(COUNT[2]==15)
					*(tdata+36+1023)='@';
			}
			memcpy(tdata+(Len-24), Data+40, 24); //tack on 2nd half
			memcpy(Data, tdata, Len);  //cpy finished buf to correct ptr

			*testNum=Tests_Four[Index_Four];
			*testCount=COUNT[2];
			if(COUNT[2]==15) //set to 9 or more
			{
				printf("************PKT4(ID Protection) [USER FQDN -odd '@'] tests complete.************\n");
				Index_Four++;
				COUNT[2]=0;
			}
			return 1;
		case 2:  /*pkt4(ID protetion)-mess with FQDN and len*/
			if(COUNT[2]==0)
				printf("******PKT4(ID Protection) [FQDN with big len] tests Beginning.******\n");	
			COUNT[2]++;
			///resize pkt and reset ptrs
			x4_buf_resize(Pkt, ((2*COUNT[2])+65) );
			Data=Pkt->data;
			Len=Pkt->len;
			*((uint32 *)Data+6)=htonl(Len);  //fix total len
			*((uint16 *)Data+15)=htons(Len-52);  //fix total len
			*(Data+32)=0x02; //set ID type to FQDN
			//fill ID data with extending ascii stuff	
			tdata=malloc(Len); //tmp data holding
			memcpy(tdata, Data, 36); //cpy up to before insert			
			for(i=0;i<(COUNT[2]);i++) //fill with A's
				*(tdata+36+i)=0x41;
			*(tdata+36+i)='.';
			for(i=0;i<(COUNT[2]);i++) //fill with B's
				*(tdata+37+COUNT[2]+i)=0x42;
			memcpy(tdata+37+COUNT[2]+i, ".com", 4);
			memcpy(tdata+(Len-24), Data+40, 24); //tack on 2nd half
			memcpy(Data, tdata, Len);  //cpy finished buf to correct ptr

			*testNum=Tests_Four[Index_Four];
			*testCount=COUNT[2];
			if(COUNT[2]==100) //set to 100 or more
			{
				printf("************PKT4(ID Protection) [FQDN with big len] tests complete.************\n");
				Index_Four++;
				COUNT[2]=0;
			}
			return 1;
		case 3:  /*pkt4(ID protetion)-randomizing of ID payload data (no len change)*/
			if(COUNT[2]==0)
				printf("******PKT4(ID Protection) [RAND ID data] tests Beginning.******\n");	
			COUNT[2]++;
			for(i=0;i<8;i++)
				*(Data+32+i)=(rand() & 0xff);	
			*testNum=Tests_Four[Index_Four];
			*testCount=COUNT[2];
			if(COUNT[2]==100) //set to 100 or more
			{
				printf("************PKT4(ID Protection) [RAND ID data] tests complete.************\n");
				Index_Four++;
				COUNT[2]=0;
			}
			return 1;
		case 4:  /*pkt4(ID protetion)-fuzzing of ID payload with len fixup*/
			if(COUNT[2]==0)
				printf("******PKT4(ID Protection) [INC ID data] tests Beginning.******\n");	
			COUNT[2]++;
			///resize pkt and reset ptrs
			x4_buf_resize(Pkt, ((16*COUNT[2])+64) );
			Data=Pkt->data;
			Len=Pkt->len;
			///fix total len
			*((uint32 *)Data+6)=htonl(Len);
			
			*((uint16 *)Data+15)=htons(Len-28-24); //fix ID len
			tdata=malloc(Len);
			if( (rand()&0xff) > 120 )  //Add some A's to IP addr
			{
				memcpy(tdata, Data, 40);			
				memset(tdata+40, 0x41, Len-64); 
			}
			else  //fill ID payload with random data
			{
				memcpy(tdata, Data, 32);			
				for(i=0;i<(Len-56);i++)
					*(tdata+32+i)=(rand() & 0xff);
			}
			memcpy(tdata+(Len-24), Data+40, 24); 
			memcpy(Data, tdata, Len);
			free(tdata);
			
			*testNum=Tests_Four[Index_Four];
			*testCount=COUNT[2];
			if(COUNT[2]==100)  //set to 100
			{
				printf("************PKT4(ID Protection) [INC ID data] tests complete.************\n");
				Index_Four++;
				COUNT[2]=0;
			}
			return 1;
		case 5:  /*pkt4(ID protetion)-fuzzing of hash payload with len fixup*/
			if(COUNT[2]==0)
				printf("******PKT4(ID Protection) [INC hash data] tests Beginning.******\n");	
			COUNT[2]++;
			///resize pkt and reset ptrs
			x4_buf_resize(Pkt, ((16*COUNT[2])+64) );
			Data=Pkt->data;
			Len=Pkt->len;
			///fix total len
			*((uint32 *)Data+6)=htonl(Len);
			*((uint16 *)Data+21)=htons(Len-28-12); //fix hash len
			for(i=0;i<(Len-64);i++)
				*(Data+64+i)=(rand() & 0xff);
			*testNum=Tests_Four[Index_Four];
			*testCount=COUNT[2];
			if(COUNT[2]==100)  //set to 100
			{
				printf("************PKT4(ID Protection) [INC hash data] tests complete.************\n");
				Index_Four++;
				COUNT[2]=0;
			}
			return 1;
		default:
			return 0;	
	}
     default:
	return -1;
   }
 }
 else if(xchg->et == 32)
 {
   switch(xchg->seqno)
   {
     case 0:
	switch(Tests_ph2_Zero[Index_ph2_Zero])
	{
		case 1:  /*ph2_pkt0, fuzz hash data*/
			if(COUNT_ph2[0]==0)
				printf("******PH2:PKT0 [INC hash data] tests Beginning.******\n");	
			COUNT_ph2[0]++;
			///resize pkt and reset ptrs
			x4_buf_resize(Pkt, ((16*COUNT_ph2[0])+292) );
			Data=Pkt->data;
			Len=Pkt->len;
			*((uint32 *)Data+6)=htonl(Len); //fix total Len
			*((uint16 *)Data+15)=htons(Len-268); //fix hash len
			tdata=malloc(Len); //tmp data holding
			memcpy(tdata, Data, 52); //cpy up to before insert			
			for(i=0;i<(Len-292);i++) //fill with rand to total size
				*(tdata+52+i)=(rand() & 0xff);
			memcpy(tdata+(Len-240), Data+52, 240); //tack on 2nd half
			memcpy(Data, tdata, Len);  //cpy finished buf to correct ptr
			free(tdata);  //free temp buf

			*testNum=Tests_ph2_Zero[Index_ph2_Zero];
			*testCount=COUNT_ph2[0];
			if(COUNT_ph2[0]==100)  //set to 100
			{
				printf("************PH2:PKT0 [INC hash data] tests complete.************\n");
				Index_ph2_Zero++;
				COUNT_ph2[0]=0;
			}
			return 1;
		case 2:  /*ph2_pkt0, fuzz first 1/3 of SA*/
			if(COUNT_ph2[0]==0)
				printf("******PH2:PKT0 [fuzz SA payload] tests Beginning.******\n");	
			COUNT_ph2[0]++;
			///resize pkt and reset ptrs
			x4_buf_resize(Pkt, ((16*COUNT_ph2[0])+292) );
			Data=Pkt->data;
			Len=Pkt->len;
			*((uint32 *)Data+6)=htonl(Len); //fix total Len
			*((uint16 *)Data+27)=htons(Len-244); //fix SA len
			tdata=malloc(Len); //tmp data holding
			memcpy(tdata, Data, 64); //cpy up to before insert			
			for(i=64;i<(Len-192);i++) //fill with rand to total size
				*(tdata+i)=(rand() & 0xff);
			memcpy(tdata+i, Data+100, 192); //tack on 2nd half
			memcpy(Data, tdata, Len); //cpy finished buf to correct ptr
			free(tdata);  //free temp buf

			*testNum=Tests_ph2_Zero[Index_ph2_Zero];
			*testCount=COUNT_ph2[0];
			if(COUNT_ph2[0]==100)  //set to 100
			{
				printf("************PH2:PKT0 [fuzz SA payload] tests complete.************\n");
				Index_ph2_Zero++;
				COUNT_ph2[0]=0;
			}
			return 1;
		case 3:  /*ph2_pkt0, fuzz first 2/3 of SA-proposal*/
			if(COUNT_ph2[0]==0)
				printf("******PH2:PKT0 [fuzz Porposal payload] tests Beginning.******\n");
			COUNT_ph2[0]++;
			///resize pkt and reset ptrs
			x4_buf_resize(Pkt, ((16*COUNT_ph2[0])+292) );
			Data=Pkt->data;
			Len=Pkt->len;
			*((uint32 *)Data+6)=htonl(Len); //fix total Len
			*((uint16 *)Data+27)=htons(Len-244); //fix SA len
			*((uint16 *)Data+33)=htons(Len-256); //fix Proposal len
			tdata=malloc(Len); //tmp data holding
			memcpy(tdata, Data, 76); //cpy up to before insert			
			for(i=76;i<(Len-192);i++) //fill with rand to total size
				*(tdata+i)=(rand() & 0xff);
			memcpy(tdata+i, Data+100, 192); //tack on 2nd half
			memcpy(Data, tdata, Len); //cpy finished buf to correct ptr
			free(tdata);  //free temp buf

			*testNum=Tests_ph2_Zero[Index_ph2_Zero];
			*testCount=COUNT_ph2[0];
			if(COUNT_ph2[0]==100)  //set to 100
			{
				printf("************PH2:PKT0 [fuzz Proposal payload] tests complete.************\n");
				Index_ph2_Zero++;
				COUNT_ph2[0]=0;
			}
			return 1;
		case 4:  /*ph2_pkt0, fuzz first 3/3 of SA*/
			if(COUNT_ph2[0]==0)
				printf("******PH2:PKT0 [fuzz Transform payload] tests Beginning.******\n");	
			COUNT_ph2[0]++;
			///resize pkt and reset ptrs
			x4_buf_resize(Pkt, ((16*COUNT_ph2[0])+292) );
			Data=Pkt->data;
			Len=Pkt->len;
			*((uint32 *)Data+6)=htonl(Len); //fix total Len
			*((uint16 *)Data+27)=htons(Len-244); //fix SA len
			*((uint16 *)Data+33)=htons(Len-256); //fix Proposal len
			*((uint16 *)Data+39)=htons(Len-268); //fix transform len
			tdata=malloc(Len); //tmp data holding
			memcpy(tdata, Data, 100); //cpy up to before insert			
			for(i=100;i<(Len-192);i++) //fill with rand to total size
				*(tdata+i)=(rand() & 0xff);
			memcpy(tdata+i, Data+100, 192); //tack on 2nd half
			memcpy(Data, tdata, Len); //cpy finished buf to correct ptr
			free(tdata);  //free temp buf

			*testNum=Tests_ph2_Zero[Index_ph2_Zero];
			*testCount=COUNT_ph2[0];
			if(COUNT_ph2[0]==100)  //set to 100
			{
				printf("************PH2:PKT0 [fuzz Transform payload] tests complete.************\n");
				Index_ph2_Zero++;
				COUNT_ph2[0]=0;
			}
			return 1;
		case 5:  /*ph2_pkt0, Nonce payload*/
			if(COUNT_ph2[0]==0)
				printf("******PH2:PKT0 [Nonce] tests Beginning.******\n");	
			COUNT_ph2[0]++;
			///resize pkt and reset ptrs
			x4_buf_resize(Pkt, ((16*COUNT_ph2[0])+292) );
			Data=Pkt->data;
			Len=Pkt->len;
			*((uint32 *)Data+6)=htonl(Len); //fix total Len
			*((uint16 *)Data+51)=htons(Len-256); //fix nonce len
			tdata=malloc(Len); //tmp data holding
			memcpy(tdata, Data, 136); //cpy up to before insert			
			for(i=136;i<(Len-156);i++) //fill with rand to total size
				*(tdata+i)=(rand() & 0xff);
			memcpy(tdata+i, Data+136, 156); //tack on 2nd half
			memcpy(Data, tdata, Len); //cpy finished buf to correct ptr
			free(tdata);  //free temp buf

			*testNum=Tests_ph2_Zero[Index_ph2_Zero];
			*testCount=COUNT_ph2[0];
			if(COUNT_ph2[0]==100)  //set to 100
			{
				printf("************PH2:PKT0 [fuzz Nonce payload] tests complete.************\n");
				Index_ph2_Zero++;
				COUNT_ph2[0]=0;
			}
			return 1;
		case 6:  /*ph2_pkt0, Key Exchange payload*/
			if(COUNT_ph2[0]==0)
				printf("******PH2:PKT0 [Key Exchange] tests Beginning.******\n");	
			COUNT_ph2[0]++;
			///resize pkt and reset ptrs
			x4_buf_resize(Pkt, ((16*COUNT_ph2[0])+292) );
			Data=Pkt->data;
			Len=Pkt->len;
			*((uint32 *)Data+6)=htonl(Len); //fix total Len
			*((uint16 *)Data+69)=htons(Len-160); //fix KE len
			tdata=malloc(Len); //tmp data holding
			memcpy(tdata, Data, 268); //cpy up to before insert			
			for(i=268;i<(Len-24);i++) //fill with rand to total size
				*(tdata+i)=(rand() & 0xff);
			memcpy(tdata+i, Data+268, 24); //tack on 2nd half
			memcpy(Data, tdata, Len); //cpy finished buf to correct ptr
			free(tdata);  //free temp buf

			*testNum=Tests_ph2_Zero[Index_ph2_Zero];
			*testCount=COUNT_ph2[0];
			if(COUNT_ph2[0]==100)  //set to 100
			{
				printf("************PH2:PKT0 [fuzz Key Exchange payload] tests complete.************\n");
				Index_ph2_Zero++;
				COUNT_ph2[0]=0;
			}
			return 1;
		case 7:  /*ph2_pkt0, ID1 payload*/
			if(COUNT_ph2[0]==0)
				printf("******PH2:PKT0 [ID1] tests Beginning.******\n");	
			COUNT_ph2[0]++;
			///resize pkt and reset ptrs
			x4_buf_resize(Pkt, ((16*COUNT_ph2[0])+292) );
			Data=Pkt->data;
			Len=Pkt->len;
			*((uint32 *)Data+6)=htonl(Len); //fix total Len
			*((uint16 *)Data+135)=htons(Len-280); //fix ID1 len
			tdata=malloc(Len); //tmp data holding
			memcpy(tdata, Data, 272 ); //cpy up to before insert			
			for(i=272;i<(Len-12);i++) //fill with rand to total size
				*(tdata+i)=(rand() & 0xff);
			memcpy(tdata+i, Data+280, 12); //tack on 2nd half
			memcpy(Data, tdata, Len); //cpy finished buf to correct ptr
			free(tdata);  //free temp buf

			*testNum=Tests_ph2_Zero[Index_ph2_Zero];
			*testCount=COUNT_ph2[0];
			if(COUNT_ph2[0]==100)  //set to 100
			{
				printf("************PH2:PKT0 [fuzz ID1 payload] tests complete.************\n");
				Index_ph2_Zero++;
				COUNT_ph2[0]=0;
			}
			return 1;
		case 8:  /*ph2_pkt0, ID2 payload*/
			if(COUNT_ph2[0]==0)
				printf("******PH2:PKT0 [ID2] tests Beginning.******\n");	
			COUNT_ph2[0]++;
			///resize pkt and reset ptrs
			x4_buf_resize(Pkt, ((16*COUNT_ph2[0])+292) );
			Data=Pkt->data;
			Len=Pkt->len;
			*((uint32 *)Data+6)=htonl(Len); //fix total Len
			*((uint16 *)Data+141)=htons(Len-280); //fix ID2 len
			for(i=0;i<(Len-292);i++)
				*(Data+292+i)=(rand() & 0xff);  //tack some rand data on end

			*testNum=Tests_ph2_Zero[Index_ph2_Zero];
			*testCount=COUNT_ph2[0];
			if(COUNT_ph2[0]==100) //set to 100
			{
				printf("************PH2:PKT0 [fuzz ID2 payload] tests complete.************\n");
				Index_ph2_Zero++;
				COUNT_ph2[0]=0;
			}
			return 1;
		case 9:  /*ph2 pkt0 - Randomize 4 bytes*/
			if(COUNT_ph2[0]==0)
				printf("******PH2:PKT0 [byte flip] tests Beginning.******\n");	
			COUNT_ph2[0]++;
			*(Data+(rand()&42))=rand()&0xff;
			*(Data+(rand()&92))=rand()&0xff;
			*(Data+(rand()&255))=rand()&0xff;
			*(Data+((rand()&255)+36))=rand()&0xff;
			*testNum=Tests_ph2_Zero[Index_ph2_Zero];
			*testCount=COUNT_ph2[0];
			if(COUNT_ph2[0]==200)  //set to 200
			{
				printf("************PH2:PKT0 [byte flip] tests complete.************\n");
				Index_ph2_Zero++;
				COUNT_ph2[0]=0;
			}
			return 1;
		default:
			return 0;	
	}
     case 2:
	switch(Tests_ph2_Two[Index_ph2_Two])
	{
		case 1:  /*pkt2, Increment hash data and len*/
			if(COUNT_ph2[1]==0)
				printf("******PH2:PKT2(HASH) [INC hash data] tests Beginning.******\n");	
			COUNT_ph2[1]++;
			///resize pkt and reset ptrs
			x4_buf_resize(Pkt, ((16*COUNT_ph2[1])+52));
			Data=Pkt->data;
			Len=Pkt->len;
			///fix total len
			*((uint32 *)Data+6)=htonl(Len);
			*((uint16 *)Data+15)=htons(Len-28); //fix hash len
			for(i=52;i<Len;i++)
				*(Data+i)=(rand() & 0xff);
			*testNum=Tests_ph2_Two[Index_ph2_Two];
			*testCount=COUNT_ph2[1];
			if(COUNT_ph2[1]==100)  //set to 100
			{
				printf("************PH2:PKT2(HASH) [INC hash data] tests complete.************\n");
				Index_ph2_Two++;
				COUNT_ph2[1]=0;
			}
			return 1;
		case 2:  /*pkt2, randomize all pkt data*/
			if(COUNT_ph2[1]==0)
				printf("******PH2:PKT2(HASH) [Randomize pkt data] tests Beginning.******\n");	
			COUNT_ph2[1]++;
			///resize pkt and reset ptrs
			x4_buf_resize(Pkt, ((16*COUNT_ph2[1])+52));
			Data=Pkt->data;
			Len=Pkt->len;
			for(i=(rand()&32);i<Len;i++)
				*(Data+i)=(rand() & 0xff);
			*testNum=Tests_ph2_Two[Index_ph2_Two];
			*testCount=COUNT_ph2[1];
			if(COUNT_ph2[1]==100)  //set to 100
			{
				printf("************PH2:PKT2(HASH) [Randomize pkt data] tests complete.************\n");
				Index_ph2_Two++;
				COUNT_ph2[1]=0;
	
			}
			return 1;
		case 3:  /*Exit*/
			printf("\n------------------IKE fuzzer complete.  Any luck?------------------\n\n");
     			exit(0);
		default:
			return 0;
	}
     default:
	return -1;
   }
 }
 else
   return -2; 
}
